/*
Copyright (C) 1992,1993,1994,1995 Trusted Information Systems, Inc.

Export of this software from the United States of America or
Canada requires a specific license from the United States
Government.  This version of this software is not suitable for
export.

WITHIN THAT CONSTRAINT, the full text of the license agreement
that specifies the conditions under which this software may be
used is published in the file license.txt in the same directory
as that containing the TIS/MOSS source.

Trusted Information Systems makes no representation about the
suitability of this software for any purpose.  It is provided
"as is" without express or implied warranty.
*/



#include "config.h"
#include <ctype.h>
#include "general.h"

#include "bbuf.h"
#include "cbio.h"
#include "crypto.h"
#include "dname.h"
#include "new.h"
#include "util.h"


/* Create a public key pair or self-signed-certificate/key pair based
 *  on the information in the supplied template and return a new user
 *  record containing the new pair and other information from the
 *  template.  
 */

char **moss_key(template, iocbs)
char **template;
struct cbstruct *iocbs;
{
    struct bbuf  *pubkey = NULLBB;
    struct bbuf  *privkey = NULLBB;
    struct bbuf  *pass = NULLBB;
    struct bbuf  *bbuf = NULLBB;
    struct bbuf  bb;

    char  **user = NULLVP; 
    char  **retuser = NULLVP; 
    char  **tmp = NULLVP;
    char  **av = NULLVP;
    char  *cp = NULLCP;
    char  *pu = NULLCP;
    char  **serial;
    char  **dn;
	    

    int missing = 0;
    int havesubject = 0;
    int haveserial = 0;

    long   validperiod = 0L;

    /* check args */

    if (template == NULLVP || iocbs == NULL_IOCBS)
	goto cleanup;

    /* we must have private-key and private-key-access tags */

    if (tag_user(template, "private-key-access") == NULLVP) {
	(void) cp2cb(iocbs->out_errs, "Missing private-key-access.\n");
	missing++;
    }

    if (tag_user (template, "private-key") == NULLVP) {
	(void) cp2cb(iocbs->out_errs, "Missing private-key.\n");
	missing++;
    }
    
    if (missing) 
	goto cleanup;

    /* see if user matching template, sans subject-name and serial-number
       already exists in the database.  If so, use existing public key.
     */


    for (av=template; av!=NULLVP && *av!=NULLCP && *(av+1)!=NULLCP; av+=2) 
	if (!strcasecmp(*av,"subject-name"))
	    havesubject++;
	else if (!strcasecmp(*av,"serial-number"))
	    haveserial++;
	else {
	    tmp = add2av(tmp, add2cp(NULLCP, *av));
	    tmp = add2av(tmp, add2cp(NULLCP, *(av+1)));
	}

    if (havesubject && haveserial) {

	rewind_indexfile();
	user = get_tvt_user(tmp);
	FREE_AV(tmp);
	
	if (user != NULLVP) {

	    /* Found a match */

	    pu = pretty_user(user);
	    cp = add2cp(NULLCP, "Adding certificate to ");
	    cp = add2cp(cp, pu);
	    FREE(pu);
	    cp = add2cp(cp, ".\n");
	    (void) cp2cb(iocbs->out_user, cp);
	    FREE(cp);
	    
	    if (tag_user(user, "certificate") != NULLVP) {
		(void) cp2cb(iocbs->out_errs,
			     "User record already has a certificate.\n");
		goto cleanup;
	    }
	
	    if ((av = tag_user (user, "public-key")) == NULLVP) {
		(void) cp2cb(iocbs->out_errs,
			     "User record does not contain a public-key.\n");
		goto cleanup;
	    }

	    bb.data = (unsigned char *) *(av+1);
	    bb.length = strlen(*(av+1));
	    if (bdecode(&bb, &pubkey) != OK) {
		(void) cp2cb(iocbs->out_errs,
			     "Unable to decode user's public key.\n");
		goto cleanup;
	    }
	    
	    if (moss_getkey(user, &privkey, iocbs)  != OK)
		goto cleanup;
	}
    }

    /* Merge template into existing user record or nothing */

    user = merge_user(user, template);

    /* create the key pair */

    if (privkey == NULLBB) {
	
	struct algent  *ae = NULL_ALGENT;
	int             key_len;
	
	/* Make sure we have all parameters for generating a key */
	
	if ((av = tag_user(user_tailor, "key-alg")) == NULLVP) {
	    (void) cp2cb(iocbs->out_errs, "Missing key-alg.\n");
	    goto cleanup;
	}
	
	if ((ae = getalgstr(algorithms, *(av+1), IK|SIG)) == NULL_ALGENT) {
	    cp = add2cp(NULLCP, *(av+1));
	    cp = add2cp(cp, " is an unknown key-alg.\n");
	    (void) cp2cb(iocbs->out_errs, cp);
	    FREE(cp);
	    goto cleanup;
	}
	
	if ((av = tag_user(user_tailor, "key-len")) == NULLVP) {
	    (void) cp2cb(iocbs->out_errs, "Missing key-len.\n");
	    goto cleanup;
	}
	
	key_len = atoi(*(av+1));
	if (key_len < ae->minlen || key_len > ae->maxlen) {
	    cp = add2cp(NULLCP, *(av+1));
	    cp = add2cp(cp, " is an invalid key-len.\n");
	    (void) cp2cb(iocbs->out_errs, cp);
	    FREE(cp);
	    goto cleanup;
	}	
	
	/* get private key password */
	
	pass = get_password(iocbs, 1);
	
	(void) cp2cb(iocbs->out_user, 
		     "Generating key pair.  This may take some time...\n");
	
	if (gen_key(ae->code, key_len, &pubkey, &privkey) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to generate public/private key pair.\n");
	    goto cleanup;
	}
	
	(void) cp2cb(iocbs->out_user, "Finished generating key pair.\n");
	
	/* add the public/private keys to the user record */
	
	if (bencode(pubkey, &bbuf) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to encode public-key for user record.\n");
	    goto cleanup;
	}
	user = add2av(user, add2cp(NULLCP, "public-key"));
	user = add2av(user, (char *)bbuf -> data);
	
	if (sav_key(user, pass, privkey) != OK) {
	    (void) cp2cb(iocbs->out_errs, "Unable to save private key.\n");
	    goto cleanup;
	}
    }

    /* create the certificate if that's what we're doing */

    if ((serial = tag_user(user, "serial-number")) != NULLVP
	&& (dn = tag_user(user, "subject-name")) != NULLVP) {
	
	char  **tmpuser = NULLVP;
	long  beginvalid, endvalid;
	struct bbuf *cert = NULLBB;

        (void) cp2cb(iocbs->out_user, "Creating self-signed certificate.\n");

	/* fill in all the pieces */

	cert = alloc_bbuf();

	if ((bbuf = hex2bin(*(serial+1))) == NULLBB
	    || serial_cert(&cert, &bbuf, WRITE) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to insert serial-number in certificate.\n");
	    goto cleanup;
	}
	FREE_BBUF(bbuf);

	if (version_cert(&cert, &cert_version, WRITE) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to insert version in certificate.\n");
	    goto cleanup;
	}
	
	if ((bbuf = str2dn(*(dn+1))) == NULLBB
	    || issuer_cert(&cert, &bbuf, WRITE) != OK
	    || subject_cert(&cert, &bbuf, WRITE) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to insert subject-name in certificate.\n");
	    goto cleanup;
	}
	FREE_BBUF(bbuf);

	if ((av = tag_user(user_tailor, "cert-period")) != NULLVP) 
	    validperiod = str2valid(*(av+1));
	beginvalid = TIME();
	endvalid = beginvalid + validperiod;
	if (valid_cert(&cert, &beginvalid, &endvalid, WRITE) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to insert validity period in certificate.\n");
	    goto cleanup;
	}

	if (key_cert(&cert, &pubkey, WRITE) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to insert public-key in certificate.\n");
	    goto cleanup;
	}

	/* self-sign the certificate */

	if (sign_cert(&cert, privkey) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to self-sign the certificate.\n");
	    goto cleanup;
	}

	/* printably encode the certificate */

	if (bencode(cert, &bbuf) != OK) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to encode certificate for user record.\n");
	    goto cleanup;
	}

	/* explode the certificate and merge the results into user */

	if ((tmpuser = explode_cert((char *)bbuf -> data)) == NULLVP ||
	(user = merge_user(user, tmpuser)) == NULLVP) {
	    (void) cp2cb(iocbs->out_errs,
			 "Unable to merge new certificate information.\n");
	    goto cleanup;
	}

        (void) cp2cb(iocbs->out_user, "Finished creating certificate.\n");

	FREE_AV(tmpuser);
	FREE_BBUF(bbuf);
	FREE_BBUF(cert);
    }

    /* all done; create the new user record to return */

    user = merge_user(template, user);
    retuser = user;
    user = NULLVP;

 cleanup:

    FREE_AV(user);
    FREE_BBUF(privkey);
    FREE_BBUF(pubkey);

    return retuser;
}
